--[[ borlox 2009-08-06T10:24:20Z 5d0b787 ]]--
local CombatDnd = LibStub("AceAddon-3.0"):NewAddon("CombatDnd", "AceEvent-3.0", "AceConsole-3.0")
_G.CombatDnd = CombatDnd	-- debugging, later it can be accessed by LibStub("AceAddon-3.0"):GetAddon("CombatDnd")

local AceDB = LibStub("AceDB-3.0")
local AceCfgReg = LibStub("AceConfigRegistry-3.0")
local AceCfgDlg = LibStub("AceConfigDialog-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("CombatDnd")

local SendChatMessage = SendChatMessage	-- because DND is a chat message...
local UnitExists = UnitExists
local UnitClassification = UnitClassification
local UnitName = UnitName
local UnitIsDND, UnitIsAFK = UnitIsDND, UnitIsAFK

local BuildOptions	-- call this AFTER the db has been created, and only once

local blizOptions
local function ChatCmd(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(blizOptions)
	else
		if input:trim() == "help" then
			input = ""
		end
		AceCfgCmd.HandleCommand(CombatDnd, "CombatDnd", appName, input)
	end
end

local MARKED_MATCH = MARKED_DND:gsub("%%s%.",".*")
local CLEARED_MATCH = CLEARED_DND

local filterNextMsg = false
local function MsgFilter(frame, event, arg1)
	if not filterNextMsg then return false end
	
	if arg1:find(MARKED_MATCH) or arg1:find(CLEARED_MATCH) then
		filterNextMsg = false
		return true -- filter this message
	end
	print("Not filtering", arg1)
	return false
end

function CombatDnd:OnInitialize()
	local defaults = {
		profile = {
			enabled = true,
			text = L["DEFAULT_TEXT"],
			replaceAFK = true,
			hideMessage = true,
			onlyBoss = false,
			onlyElite = false,
		},
	}
	self.db = AceDB:New("CombatDndDB", defaults)
	
	AceCfgReg:RegisterOptionsTable("CombatDnd", BuildOptions())
	blizOptions = AceCfgDlg:AddToBlizOptions("CombatDnd")
	
	self:RegisterChatCommand("combatdnd", ChatCmd)
	self:RegisterChatCommand("cdnd", ChatCmd)
	
	self:SetEnabledState(self.db.profile.enabled)
end

function CombatDnd:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", MsgFilter)
end

function CombatDnd:OnDisable()
	if UnitIsDND("player") and self.changeDnd then	-- remove DND if it's set by this addon
		self:SetDND(false)
	end
	
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", MsgFilter)
end

function CombatDnd:SetDND(set, text)
	filterNextMsg = self.db.profile.hideMessage
	if set then
		SendChatMessage(text, "DND")
	else
		SendChatMessage("", "DND")
	end
end

local function CheckUnit(unit, db)
	local class = UnitExists(unit) and UnitClassification(unit) or "normal"
	local name  = UnitExists(unit) and UnitName(unit)           or "Unknown"
	
	if db.onlyBoss and (class ~= "worldboss") then 
		return false, name 
	end
	
	if db.onlyElite and ((class ==  "normal") or (class == "rare")) then 
		return false, name 
	end
	
	return true, name
end

function CombatDnd:PLAYER_REGEN_DISABLED()
	local db = self.db.profile
	
	if UnitIsDND("player") then return end
	if not db.replaceAFK and UnitIsAFK("player") then return end
	
	local result, name
	if UnitIsFriend("target", "player") then
		result, name = CheckUnit("targettarget", db)
	else
		result, name = CheckUnit("target", db)
	end
	if not result then return end
	
	local text = db.text:gsub("{N}", name)
	self:SetDND(true, text)
	self.changeDnd = 1
end

function CombatDnd:PLAYER_REGEN_ENABLED()
	if UnitIsDND("player") and self.changeDnd then
		self:SetDND(false)
		self.changeDnd = nil
	end
end

do
	local db
	local function DefaultSet(info, value)
		db.profile[info.arg or info[#info]] = value
	end
	
	local function DefaultGet(info)
		return db.profile[info.arg or info[#info]]
	end
	
	local function SetEnabled(info, value)
		db.profile.enabled = value
		if value then
			CombatDnd:Enable()
		else
			CombatDnd:Disable()
		end
	end
	
	local function GetEnabled(info)
		return db.profile.enabled
	end
	
	local function SetText(info, value)
		if value == "" or value == "DEFAULT" then
			value = L["DEFAULT_TEXT"]
		end
		db.profile.text = value
		CombatDnd:Print(L["Changed the auto dnd text to %s."]:format(value))
	end
	
	function BuildOptions()
		db = CombatDnd.db
		
		return {
			name    = "CombatDnd",
			type    = "group",
			handler = CombatDnd,
			get     = DefaultGet,
			set     = DefaultSet,
			args    = {
				text = {
					name  = L["Text"],
					desc  = L["Sets the auto dnd text."],
					usage = L["<text>"],
					type  = "input",
					order = 3,
					set   = SetText,			
				},
				toggles = {
					type   = "group",
					name   = "", -- anonymus subgroup
					inline = true,
					order  = 4,
					args = {
						replaceAFK = {
							name  = L["Replace AFK"],
							desc  = L["Replace your AFK status when entering combat."],
							type  = "toggle",
						},
						hideMessage = {
							name = L["Hide DND message"],
							desc = L["Hide the \"You are marked DND...\" message."],
							type = "toggle",
						},
						onlyBoss = {
							name  = L["Only Boss"],
							desc  = L["Only set DND when your target is a boss."],
							type  = "toggle",
						},
						onlyElite = {
							name  = L["Only Elite"],
							desc  = L["Only set DND when your target is an elite mob."],
							type  = "toggle",
						},
					},
				},
				enable = {
					name  = L["Enable"],
					type  = "toggle",
					order = 1,
					set   = SetEnabled,
					get   = GetEnabled,
				},
			}
		}
	end
end
